@@ -1,5 +1,5 @@
-/* to add back later: [ [ R c p p :: d e p e n d s(s f)]] */
-// to add back later: #include <sf.h>
+// [[Rcpp::depends(sf)]]
+#include <sf.h>
 
 #include <Rcpp.h>
 
@@ -7,9 +7,26 @@
 
 extern "C" {
 #include <liblwgeom.h>
+#ifdef HAVE_LIBLWGEOM_INTERNAL_H
+# include <liblwgeom_internal.h>
+#else /* hard copy from liblwgeom_internal.h: */
+# ifndef NO_GRID_IN_PLACE
+typedef struct gridspec_t
+{
+	double ipx;
+	double ipy;
+	double ipz;
+	double ipm;
+	double xsize;
+	double ysize;
+	double zsize;
+	double msize;
+}
+gridspec;
+void lwgeom_grid_in_place(LWGEOM *lwgeom, const gridspec *grid);
+# endif
+#endif
 }
-
-#include "wkb.h"
 
 #include "lwgeom.h"
 
@@ -23,8 +40,7 @@ Rcpp::CharacterVector CPL_lwgeom_version(bool b = false) {
 // in
 std::vector<LWGEOM *> lwgeom_from_sfc(Rcpp::List sfc) {
 	std::vector<LWGEOM *> lwgeom_v(sfc.size()); // return
-	// Rcpp::List wkblst = sf::CPL_write_wkb(sfc, true);
-	Rcpp::List wkblst = CPL_write_wkb(sfc, true);
+	Rcpp::List wkblst = sf::CPL_write_wkb(sfc, true); // true: write EWKB, puts EPSG inside the wkb
 	for (int i = 0; i < wkblst.size(); i++) {
 		Rcpp::RawVector rv = wkblst[i];
 		const uint8_t *wkb = &(rv[0]); 
@@ -46,8 +62,7 @@ Rcpp::List sfc_from_lwgeom(std::vector<LWGEOM *> lwgeom_v) {
 		lwfree((void *) wkb);
 		wkblst[i] = raw;
 	}
-	// return sf::CPL_read_wkb(wkblst, true, false);
-	return CPL_read_wkb(wkblst, true, false);
+	return sf::CPL_read_wkb(wkblst, true, false);
 }
 
 // [[Rcpp::export]]
@@ -166,3 +181,97 @@ Rcpp::List CPL_subdivide(Rcpp::List sfc, int max_vertices = 256) {
 		lwgeom_v[i] = lwcollection_as_lwgeom(lwgeom_subdivide(lwgeom_v[i], max_vertices));
 	return sfc_from_lwgeom(lwgeom_v);
 }
+
+// [[Rcpp::export]]
+Rcpp::List CPL_snap_to_grid(Rcpp::List sfc, Rcpp::NumericVector origin, Rcpp::NumericVector size) {
+#ifdef NO_GRID_IN_PLACE
+	Rcpp::stop("st_snap_to_grid: not supported in this version of liblwgeom\n"); // #nocov
+	// return sfc;
+#else
+	// initialize input data
+	std::vector<LWGEOM *> lwgeom_v = lwgeom_from_sfc(sfc);
+	// initialize grid
+	gridspec grid;
+	grid.ipx = origin[0];
+	grid.ipy = origin[1];
+	grid.ipz = origin[2];
+	grid.ipm = origin[3];
+	grid.xsize = size[0];
+	grid.ysize = size[1];
+	grid.zsize = size[2];
+	grid.msize = size[3];
+	// snap geometries to grid
+	for (size_t i = 0; i < lwgeom_v.size(); i++)
+		lwgeom_grid_in_place(lwgeom_v[i], &grid);
+	// return snapped geometries
+	return sfc_from_lwgeom(lwgeom_v); 
+#endif
+}
+
+// [[Rcpp::export]]
+Rcpp::NumericVector CPL_perimeter(Rcpp::List sfc, bool do2d = false) {
+	Rcpp::NumericVector out(sfc.length());
+	std::vector<LWGEOM *> lwgeom_v = lwgeom_from_sfc(sfc);
+
+	if (do2d) {
+		for (size_t i = 0; i < lwgeom_v.size(); i++)
+			out[i] = lwgeom_perimeter_2d(lwgeom_v[i]);
+	} else {
+		for (size_t i = 0; i < lwgeom_v.size(); i++)
+			out[i] = lwgeom_perimeter(lwgeom_v[i]);
+	}
+	return out;
+}
+
+
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_is_polygon_cw(Rcpp::List sfc) {
+  std::vector<LWGEOM *> lwgeom_cw = lwgeom_from_sfc(sfc);
+  Rcpp::LogicalVector out(sfc.length());
+  for (size_t i = 0; i < lwgeom_cw.size(); i++) {
+    out[i] = lwgeom_is_clockwise(lwgeom_cw[i]);
+    lwgeom_free(lwgeom_cw[i]);
+  }
+  return out;
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_force_polygon_cw(Rcpp::List sfc) {
+  
+  std::vector<LWGEOM *> lwgeom_cw = lwgeom_from_sfc(sfc);
+  for (size_t i = 0; i < lwgeom_cw.size(); i++) {
+    lwgeom_force_clockwise(lwgeom_cw[i]);
+  }
+  return sfc_from_lwgeom(lwgeom_cw);
+}
+
+// [[Rcpp::export]]
+Rcpp::NumericMatrix CPL_startpoint(Rcpp::List sfc) {
+  
+  std::vector<LWGEOM *> lwgeom_cw = lwgeom_from_sfc(sfc);
+  Rcpp::NumericMatrix m(lwgeom_cw.size(), 2);
+  
+  POINT4D p;
+  for (size_t i = 0; i < lwgeom_cw.size(); i++) {
+    lwgeom_startpoint(lwgeom_cw[i], &p);
+    m(i, 0) = p.x;
+    m(i, 1) = p.y;
+  }
+  
+  return m;
+  // next step: get it into sf form
+  // return sfc_from_lwgeom(lwgeom_cw);
+}
+
+// [[Rcpp::export]]
+Rcpp::CharacterVector CPL_sfc_to_wkt(Rcpp::List sfc, Rcpp::IntegerVector precision) {
+
+  std::vector<LWGEOM *> lwgeom_cw = lwgeom_from_sfc(sfc);
+  Rcpp::CharacterVector out;
+  for (size_t i = 0; i < lwgeom_cw.size(); i++) {
+	char *wkt = lwgeom_to_wkt(lwgeom_cw[i], WKT_EXTENDED, precision[0], NULL);
+    out.push_back(wkt);
+	free(wkt);
+  }
+  return out;
+}
